@@ -1,53 +1,66 @@
-use std::{
-    collections::BTreeMap,
-    sync::mpsc::{Receiver, Sender},
-    thread::JoinHandle,
-};
+use std::{collections::BTreeMap, fmt::Display};
 
 use merge::Merge;
 
-use pallas::ouroboros::network::handshake::{MAINNET_MAGIC, TESTNET_MAGIC};
 use serde_derive::{Deserialize, Serialize};
 use strum_macros::Display;
 
-use crate::mapping::MapperConfig;
+use serde_json::Value as JsonValue;
 
 pub type Error = Box<dyn std::error::Error>;
 
 #[derive(Serialize, Deserialize, Debug, Clone)]
-pub struct ChainWellKnownInfo {
-    pub shelley_known_slot: u64,
-    pub shelley_known_hash: String,
-    pub shelley_known_time: u64,
+#[serde(rename_all = "snake_case")]
+pub enum MetadatumRendition {
+    MapJson(JsonValue),
+    ArrayJson(JsonValue),
+    IntScalar(i64),
+    TextScalar(String),
+    BytesHex(String),
 }
 
-impl ChainWellKnownInfo {
-    pub fn try_from_magic(magic: u64) -> Result<ChainWellKnownInfo, Error> {
-        match magic {
-            MAINNET_MAGIC => Ok(ChainWellKnownInfo {
-                shelley_known_slot: 4492799,
-                shelley_known_hash:
-                    "f8084c61b6a238acec985b59310b6ecec49c0ab8352249afd7268da5cff2a457".to_string(),
-                shelley_known_time: 1596059071,
-            }),
-            TESTNET_MAGIC => Ok(ChainWellKnownInfo {
-                shelley_known_slot: 1598399,
-                shelley_known_hash:
-                    "7e16781b40ebf8b6da18f7b5e8ade855d6738095ef2f1c58c77e88b6e45997a4".to_string(),
-                shelley_known_time: 1595967596,
-            }),
-            _ => Err("can't infer well-known chain infro from specified magic".into()),
+impl Display for MetadatumRendition {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        match self {
+            MetadatumRendition::MapJson(x) => x.fmt(f),
+            MetadatumRendition::ArrayJson(x) => x.fmt(f),
+            MetadatumRendition::IntScalar(x) => x.fmt(f),
+            MetadatumRendition::TextScalar(x) => x.fmt(f),
+            MetadatumRendition::BytesHex(x) => x.fmt(f),
         }
     }
 }
 
 #[derive(Serialize, Deserialize, Debug, Clone)]
 pub struct MetadataRecord {
-    pub key: String,
-    pub subkey: Option<String>,
-    // TODO: value should be some sort of structured, JSON-like value.
-    // we could use Pallas' Metadatum struct, but it needs to be clonable
-    pub value: Option<String>,
+    pub label: String,
+
+    #[serde(flatten)]
+    pub content: MetadatumRendition,
+}
+
+impl From<MetadataRecord> for EventData {
+    fn from(x: MetadataRecord) -> Self {
+        EventData::Metadata(x)
+    }
+}
+
+#[derive(Serialize, Deserialize, Debug, Clone)]
+pub struct CIP25AssetRecord {
+    pub version: String,
+    pub policy: String,
+    pub asset: String,
+    pub name: Option<String>,
+    pub image: Option<String>,
+    pub media_type: Option<String>,
+    pub description: Option<String>,
+    pub raw_json: JsonValue,
+}
+
+impl From<CIP25AssetRecord> for EventData {
+    fn from(x: CIP25AssetRecord) -> Self {
+        EventData::CIP25Asset(x)
+    }
 }
 
 #[derive(Serialize, Deserialize, Debug, Clone)]
@@ -56,6 +69,12 @@ pub struct TxInputRecord {
     pub index: u64,
 }
 
+impl From<TxInputRecord> for EventData {
+    fn from(x: TxInputRecord) -> Self {
+        EventData::TxInput(x)
+    }
+}
+
 #[derive(Serialize, Deserialize, Debug, Clone)]
 pub struct OutputAssetRecord {
     pub policy: String,
@@ -63,6 +82,12 @@ pub struct OutputAssetRecord {
     pub amount: u64,
 }
 
+impl From<OutputAssetRecord> for EventData {
+    fn from(x: OutputAssetRecord) -> Self {
+        EventData::OutputAsset(x)
+    }
+}
+
 #[derive(Serialize, Deserialize, Debug, Clone)]
 pub struct TxOutputRecord {
     pub address: String,
@@ -70,6 +95,12 @@ pub struct TxOutputRecord {
     pub assets: Option<Vec<OutputAssetRecord>>,
 }
 
+impl From<TxOutputRecord> for EventData {
+    fn from(x: TxOutputRecord) -> Self {
+        EventData::TxOutput(x)
+    }
+}
+
 #[derive(Serialize, Deserialize, Debug, Clone)]
 pub struct MintRecord {
     pub policy: String,
@@ -77,6 +108,12 @@ pub struct MintRecord {
     pub quantity: i64,
 }
 
+impl From<MintRecord> for EventData {
+    fn from(x: MintRecord) -> Self {
+        EventData::Mint(x)
+    }
+}
+
 #[derive(Serialize, Deserialize, Debug, Clone, Default)]
 pub struct TransactionRecord {
     pub fee: u64,
@@ -95,6 +132,12 @@ pub struct TransactionRecord {
     pub mint: Option<Vec<MintRecord>>,
 }
 
+impl From<TransactionRecord> for EventData {
+    fn from(x: TransactionRecord) -> Self {
+        EventData::Transaction(x)
+    }
+}
+
 #[derive(Serialize, Deserialize, Debug, Clone, Merge, Default)]
 pub struct EventContext {
     pub block_hash: Option<String>,
@@ -115,25 +158,37 @@ pub enum StakeCredential {
     Scripthash(String),
 }
 
+#[derive(Serialize, Deserialize, Debug, Clone, PartialEq, Eq, PartialOrd, Ord)]
+pub struct BlockRecord {
+    pub body_size: usize,
+    pub issuer_vkey: String,
+    pub tx_count: usize,
+    pub slot: u64,
+    pub hash: String,
+    pub number: u64,
+}
+
 #[derive(Serialize, Deserialize, Display, Debug, Clone)]
 #[serde(rename_all = "snake_case")]
 pub enum EventData {
-    Block {
-        body_size: usize,
-        issuer_vkey: String,
-        tx_count: usize,
-    },
+    Block(BlockRecord),
+    BlockEnd(BlockRecord),
     Transaction(TransactionRecord),
+    TransactionEnd(TransactionRecord),
     TxInput(TxInputRecord),
     TxOutput(TxOutputRecord),
     OutputAsset(OutputAssetRecord),
     Metadata(MetadataRecord),
+
+    #[serde(rename = "cip25_asset")]
+    CIP25Asset(CIP25AssetRecord),
+
     Mint(MintRecord),
     Collateral {
         tx_id: String,
         index: u64,
     },
-    NativeScript,
+    NativeScript {},
     PlutusScript {
         data: String,
     },
@@ -184,75 +239,3 @@ pub struct Event {
 
     pub fingerprint: Option<String>,
 }
-
-pub type PartialBootstrapResult = Result<(JoinHandle<()>, Receiver<Event>), Error>;
-
-pub type BootstrapResult = Result<JoinHandle<()>, Error>;
-
-pub trait SourceConfig {
-    fn bootstrap(&self) -> PartialBootstrapResult;
-}
-
-pub trait FilterConfig {
-    fn bootstrap(&self, input: Receiver<Event>) -> PartialBootstrapResult;
-}
-
-pub trait SinkConfig {
-    fn bootstrap(&self, input: Receiver<Event>) -> BootstrapResult;
-}
-
-#[derive(Clone, Debug)]
-pub struct EventWriter {
-    context: EventContext,
-    output: Sender<Event>,
-    chain_info: Option<ChainWellKnownInfo>,
-    pub mapping_config: MapperConfig,
-}
-
-impl EventWriter {
-    pub fn new(
-        output: Sender<Event>,
-        chain_info: Option<ChainWellKnownInfo>,
-        mapping_config: MapperConfig,
-    ) -> Self {
-        EventWriter {
-            context: EventContext::default(),
-            output,
-            chain_info,
-            mapping_config,
-        }
-    }
-
-    pub fn append(&self, data: EventData) -> Result<(), Error> {
-        let evt = Event {
-            context: self.context.clone(),
-            data,
-            fingerprint: None,
-        };
-
-        self.output.send(evt)?;
-
-        Ok(())
-    }
-
-    pub fn child_writer(&self, mut extra_context: EventContext) -> EventWriter {
-        extra_context.merge(self.context.clone());
-
-        EventWriter {
-            context: extra_context,
-            output: self.output.clone(),
-            chain_info: self.chain_info.clone(),
-            mapping_config: self.mapping_config.clone(),
-        }
-    }
-
-    pub fn compute_timestamp(&self, slot: u64) -> Option<u64> {
-        self.chain_info
-            .as_ref()
-            .map(|info| info.shelley_known_time + (slot - info.shelley_known_slot))
-    }
-}
-
-pub trait EventSource {
-    fn write_events(&self, writer: &EventWriter) -> Result<(), Error>;
-}
