@@ -1,7 +1,7 @@
 pub mod terminal;
 
-#[cfg(feature = "tuisink")]
-pub mod tui;
+#[cfg(feature = "webhook")]
+pub mod webhook;
 
 #[cfg(feature = "kafkasink")]
 pub mod kafka;
