@@ -3,8 +3,8 @@ use std::fmt::{Display, Write};
 use crossterm::style::{Attribute, Color, Stylize};
 
 use crate::framework::{
-    Event, EventData, MetadataRecord, MintRecord, OutputAssetRecord, TransactionRecord,
-    TxInputRecord, TxOutputRecord,
+    BlockRecord, CIP25AssetRecord, Event, EventData, MetadataRecord, MintRecord, OutputAssetRecord,
+    TransactionRecord, TxInputRecord, TxOutputRecord,
 };
 
 pub struct LogLine {
@@ -18,18 +18,23 @@ pub struct LogLine {
 impl LogLine {
     pub fn new(source: Event, max_width: usize) -> LogLine {
         match &source.data {
-            EventData::Block {
+            EventData::Block(BlockRecord {
                 body_size,
                 issuer_vkey,
                 tx_count,
+                slot,
+                hash,
+                number,
                 ..
-            } => {
+            }) => {
                 LogLine {
                     prefix: "BLOCK",
                     color: Color::Magenta,
                     content: format!(
-                    "{{ hash: {}, body size: {}, tx_count: {}, issuer vkey: {}, timestamp: {} }}",
-                    &source.context.block_hash.as_ref().unwrap_or(&"".to_string()),
+                    "{{ slot: {}, hash: {}, number: {}, body size: {}, tx_count: {}, issuer vkey: {}, timestamp: {} }}",
+                    slot,
+                    hash,
+                    number,
                     body_size,
                     tx_count,
                     issuer_vkey,
@@ -39,6 +44,25 @@ impl LogLine {
                     max_width,
                 }
             }
+            EventData::BlockEnd(BlockRecord {
+                slot,
+                hash,
+                number,
+                ..
+            }) => {
+                LogLine {
+                    prefix: "ENDBLK",
+                    color: Color::DarkMagenta,
+                    content: format!(
+                    "{{ slot: {}, hash: {}, number: {} }}",
+                    slot,
+                    hash,
+                    number,
+                ),
+                    source,
+                    max_width,
+                }
+            }
             EventData::Transaction(TransactionRecord {
                 total_output,
                 fee,
@@ -54,6 +78,16 @@ impl LogLine {
                 source,
                 max_width,
             },
+            EventData::TransactionEnd(TransactionRecord { .. }) => LogLine {
+                prefix: "ENDTX",
+                color: Color::DarkBlue,
+                content: format!(
+                    "{{ hash: {:?} }}",
+                    &source.context.tx_hash
+                ),
+                source,
+                max_width,
+            },
             EventData::TxInput(TxInputRecord { tx_id, index }) => LogLine {
                 prefix: "STXI",
                 color: Color::Blue,
@@ -85,13 +119,10 @@ impl LogLine {
                 source,
                 max_width,
             },
-            EventData::Metadata(MetadataRecord { key, subkey, value }) => LogLine {
+            EventData::Metadata(MetadataRecord { label, content }) => LogLine {
                 prefix: "META",
                 color: Color::Yellow,
-                content: format!(
-                    "{{ key: {}, sub key: {:?}, value: {:?} }}",
-                    key, subkey, value
-                ),
+                content: format!("{{ label: {}, content: {} }}", label, content),
                 source,
                 max_width,
             },
@@ -109,7 +140,7 @@ impl LogLine {
                 source,
                 max_width,
             },
-            EventData::NativeScript => LogLine {
+            EventData::NativeScript {} => LogLine {
                 prefix: "NATIVE",
                 color: Color::White,
                 content: "{{ ... }}".to_string(),
@@ -213,6 +244,25 @@ impl LogLine {
                 source,
                 max_width,
             },
+            EventData::CIP25Asset(CIP25AssetRecord {
+                policy,
+                asset,
+                name,
+                image,
+                ..
+            }) => LogLine {
+                prefix: "CIP25",
+                color: Color::DarkYellow,
+                content: format!(
+                    "{{ policy: {}, asset: {}, name: {}, image: {} }}",
+                    policy,
+                    asset,
+                    name.as_deref().unwrap_or("?"),
+                    image.as_deref().unwrap_or("?")
+                ),
+                source,
+                max_width,
+            },
         }
     }
 }
