@@ -1,8 +1,8 @@
-use std::{sync::mpsc::Receiver, time::Duration};
+use std::time::Duration;
 
 use serde_derive::Deserialize;
 
-use crate::framework::{BootstrapResult, Event, SinkConfig};
+use crate::pipelining::{BootstrapResult, SinkProvider, StageReceiver};
 
 use super::run::reducer_loop;
 
@@ -10,11 +10,11 @@ const THROTTLE_MIN_SPAN_MILLIS: u64 = 300;
 
 #[derive(Default, Debug, Deserialize)]
 pub struct Config {
-    throttle_min_span_millis: Option<u64>,
+    pub throttle_min_span_millis: Option<u64>,
 }
 
-impl SinkConfig for Config {
-    fn bootstrap(&self, input: Receiver<Event>) -> BootstrapResult {
+impl SinkProvider for Config {
+    fn bootstrap(&self, input: StageReceiver) -> BootstrapResult {
         let throttle_min_span = Duration::from_millis(
             self.throttle_min_span_millis
                 .unwrap_or(THROTTLE_MIN_SPAN_MILLIS),
