@@ -1,4 +1,6 @@
-use std::{net::TcpStream, ops::Deref, os::unix::net::UnixStream, sync::mpsc};
+#[cfg(target_family = "unix")]
+use std::os::unix::net::UnixStream;
+use std::{net::TcpStream, ops::Deref};
 
 use net2::TcpStreamExt;
 
@@ -13,12 +15,13 @@ use pallas::ouroboros::network::{
 use serde_derive::Deserialize;
 
 use crate::{
-    framework::{ChainWellKnownInfo, Error, EventWriter, PartialBootstrapResult, SourceConfig},
-    mapping::MapperConfig,
+    mapper::{ChainWellKnownInfo, Config as MapperConfig, EventWriter},
+    pipelining::{new_inter_stage_channel, PartialBootstrapResult, SourceProvider},
     sources::common::{find_end_of_chain, AddressArg, BearerKind, MagicArg, PointArg},
+    Error,
 };
 
-use super::observe_forever;
+use super::run::observe_forever;
 
 #[derive(Debug, Deserialize)]
 pub struct Config {
@@ -46,6 +49,7 @@ fn do_handshake(channel: &mut Channel, magic: u64) -> Result<(), Error> {
     }
 }
 
+#[cfg(target_family = "unix")]
 fn setup_unix_multiplexer(path: &str) -> Result<Multiplexer, Error> {
     let unix = UnixStream::connect(path)?;
 
@@ -60,12 +64,13 @@ fn setup_tcp_multiplexer(address: &str) -> Result<Multiplexer, Error> {
     Multiplexer::setup(tcp, &[0, 5])
 }
 
-impl SourceConfig for Config {
+impl SourceProvider for Config {
     fn bootstrap(&self) -> PartialBootstrapResult {
-        let (output_tx, output_rx) = mpsc::channel();
+        let (output_tx, output_rx) = new_inter_stage_channel(None);
 
         let mut muxer = match self.address.0 {
             BearerKind::Tcp => setup_tcp_multiplexer(&self.address.1)?,
+            #[cfg(target_family = "unix")]
             BearerKind::Unix => setup_unix_multiplexer(&self.address.1)?,
         };
 
